PLUGIN_URL_NAME_PREFIX = 'djangocms_alias'

# Alias Url endpoint names
CREATE_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_create'
DELETE_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_delete'
DETACH_ALIAS_PLUGIN_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_detach_plugin'
CATEGORY_LIST_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_category_changelist'
CHANGE_CATEGORY_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_category_change'
SELECT2_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_select2'
USAGE_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_alias_usage'
LIST_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_alias_changelist'
CHANGE_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_alias_change'
DELETE_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_alias_delete'
CATEGORY_SELECT2_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_category_list_select2'
# Static Alias
DEFAULT_STATIC_ALIAS_CATEGORY_NAME = "Static Alias"

# Admin Filters
SITE_FILTER_URL_PARAM = "site"
SITE_FILTER_NO_SITE_VALUE = "no_site"
UNPUBLISHED_FILTER_URL_PARAM = "unpublished"
CATEGORY_FILTER_PARAM = "category"
