{% extends "admin/delete_confirmation.html" %}
{% load i18n admin_urls cms_tags %}

{% block breadcrumbs %}{% endblock %}

{% block content %}
<p>
  {% blocktrans with escaped_object=object %}Are you sure you want to detach the {{ object_name }} "{{ escaped_object }}"?{% endblocktrans %}
</p>

<br/>

<form method="post">{% csrf_token %}
  <div>
    <input type="hidden" name="post" value="yes" />
    <input type="submit" value="{% trans "Yes, I'm sure" %}" />
    <a href="#" class="button cancel-link">{% trans "No, take me back" %}</a>
  </div>
</form>
{% endblock %}
