# Generated by Django 2.2.13 on 2020-07-23 14:24

from django.db import migrations, models
import django.db.models.deletion
import parler.fields


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('djangocms_alias', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='alias',
            name='creation_method',
            field=models.CharField(blank=True, choices=[('template', 'by template'), ('code', 'by code')], default='code', max_length=20, verbose_name='creation_method'),
        ),
        migrations.AddField(
            model_name='alias',
            name='site',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.Site'),
        ),
        migrations.AddField(
            model_name='alias',
            name='static_code',
            field=models.CharField(blank=True, help_text='To render the alias in templates.', max_length=255, null=True, verbose_name='static code'),
        ),
        migrations.AlterField(
            model_name='categorytranslation',
            name='master',
            field=parler.fields.TranslationsForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='djangocms_alias.Category'),
        ),
        migrations.AlterUniqueTogether(
            name='alias',
            unique_together={('static_code', 'site')},
        ),
    ]
